/*******************************************************
 *
 *  File: boot2nd.c
 *
 *  Author: Peter van Sebille (peter@yipton.demon.co.uk)
 *
 *  (c) Copyright 2001, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************/

#include "boot2nd_if.h"
#include "video.h"
#include "armlinux.h"
#include "lib.h"
#include "boot2nd.h"


/******************************************
 *
 * CRC checks
 *
 *****************************************/

#ifdef __CRC__


void crc_verify(unsigned** memory_map, int wait)
{
	unsigned*			crc_field0;
	unsigned*			crc_field1;
	char				num[32];

	video_puts("Calculating CRC checks...\n");
	
	crc_sections(memory_map, 1);
	crc_field0 = CRC_FIELD(memory_map[PAGE_NO_BOOT1ST_PARAM], 0);
	crc_field1 = CRC_FIELD(memory_map[PAGE_NO_BOOT1ST_PARAM], 1);
	
	video_puts("Kernel CRC: ");
	video_puts(itoa_hex(crc_field0[1], num));
		
	if (crc_field0[1] == crc_field1[1])
		video_puts(" ok\n");
	else
	{
		video_puts(" failure, differs from ");
		video_puts(itoa_hex(crc_field1[1], num));
		video_puts("\n");
		wait = 1;
	}
		
	video_puts("Initrd CRC: ");
	video_puts(itoa_hex(crc_field0[2], num));
		
	if (crc_field0[2] == crc_field1[2])
		video_puts(" ok\n");
	else
	{
		video_puts(" failure, differs from ");
		video_puts(itoa_hex(crc_field1[2], num));
		video_puts("\n");
		wait = 1;
	}

	while(wait)
		;
}



#endif			/* __CRC__ */


/*******************************************
 *
 * Interface functions
 *
 *******************************************/


void relocate_main(unsigned** memory_map, unsigned** section_start)
{
	boot1st_param_t* 	boot1st_param;
	unsigned**			initrd_start;
	int					initrd_size = 0;
	int					did_reloc;

		/*
		 * All the pages in our boot2nd image our now adjecent in memory, time to setup
		 * the video
		 */
	boot1st_param = (boot1st_param_t*) memory_map[PAGE_NO_BOOT1ST_PARAM];
	video_setup(boot1st_param->video_base,
				boot1st_param->video_width,
				boot1st_param->video_height,
				boot1st_param->video_bpp,
				boot1st_param->video_fg_colour,
				boot1st_param->video_bg_colour
				);
	
	video_clear();
	
	video_puts("Second stage boot loader, version 1.0 by Peter van Sebille\n");
	
		/*
		 * Relocate kernel section
		 */
	video_puts("\nRelocating the kernel");
	section_start = relocate_section(section_start, memory_map + PAGE_NO_FREE_PAGE, boot1st_param->kernel_reloc_dst, -1, &did_reloc, 4);

	if (boot1st_param->initrd_reloc_dst)
	{
    		/*
    		 * Relocate initrd section
    		 */
    	video_puts("\nRelocating initrd");
    	initrd_start = section_start;
    	section_start = relocate_section(section_start, memory_map + PAGE_NO_FREE_PAGE, boot1st_param->initrd_reloc_dst, -1, &did_reloc, 8);
    	initrd_size = ((((unsigned) section_start - (unsigned) initrd_start) >> 2) - 1 ) * PAGE_SIZE;
    	if (initrd_size < 0)
    		initrd_size = 0;
	}
	
		/*
		 * Setup the kernel parameters and relocate to its destination
		 */
	armlinux_format_paramater_page(memory_map[PAGE_NO_FREE_PAGE], boot1st_param, memory_map[PAGE_NO_BOOT_PARAM], initrd_size);
	copy_page(KERNEL_PARAM_DST(boot1st_param->kernel_reloc_dst), memory_map[PAGE_NO_FREE_PAGE]);

	video_puts("\nDone, booting the kernel...\n");
	
	CRC_VERIFY(memory_map, 1);

	_boot_kernel(boot1st_param->kernel_reloc_dst, boot1st_param->arm_arch_no);
	
	// never get here
}

